@@ -208,8 +208,19 @@ test.all <- function(p) {
  detach(ob,character.only=TRUE)
 }
 
+#lookup a set of items from a bigger set
+lookupItem <- function(x,y) {
+n.look <- NROW(x)
+possible <- list()
+y <- as.character(y)
+x <- as.character(x)
+for(i in 1:n.look){
+ temp <- grep(x[i],y)
+ if(length(temp)>0) possible[i]<- temp
 
-
+}
+return(possible)
+}
   
   
   #lookup which x's are found in y[c1],return matches for y[]
@@ -224,14 +235,14 @@ return(y)}
  #use lookup to take fa/ic output and show the results 
  #modified July 4, 2017 to allow for omega output as well
  #modified June 23, 2018 to limit to top n items per factor and abs(loading) > cut
+ #modified April 30, 2020 to include the ability to handle bassAckward output
 "fa.lookup"  <-
    function(f,dictionary=NULL,digits=2,cut=.0,n=NULL,sort=TRUE) {
-    if(sort) {f <- fa.sort(f)}
-    none <- NA 
-   if(length(class(f)) > 1){ obnames <- cs(omega, fa, principal, iclust, none)
+    omega <- bassAck <- none <- lavaan <- NA    #weird requirement to avoid being global
+   if(length(class(f)) > 1){ obnames <- cs(omega, fa, principal, iclust,bassAck, none)
      value <- inherits(f, obnames, which=TRUE)
 			   if (any(value > 1)) {value <- obnames[which(value >0)]} else {value <- "none"}}
-  
+   if(sort & (value !="bassAck")) {f <- fa.sort(f)}
    switch(value,
     
     omega = {f <- f$schmid$sl
@@ -246,19 +257,32 @@ return(y)}
               f <- f$loading},
     iclust = {f <- f$loadings
               h2 <- NULL},
+    bassAck = {nlevels <- length(f$bass.ack)	
+              h2 <- NULL
+              ord1 <- rownames(fa.sort(f$bass.ack[[nlevels-1]]))
+               f <- f$bass.ack[[nlevels]]
+               f <- f[,ord1]
+               f <- fa.sort(f)
+               colnames(f) <- paste0("F",1:ncol(f))           
+    },
     none = {f <- f
             h2 <- NULL})
     
     n.fact <- NCOL(f)
 
    ord <- rownames(f)
+   old.names <- ord
+   ord <- sub("-","",ord)
+   rownames(f) <- ord
    if(!is.null(dictionary))  { 
    contents <- lookup(rownames(f),dictionary)} else {message("fa.lookup requires a dictionary, otherwise just use fa.sort")}
    if(!is.null(h2)) {results <- data.frame(round(unclass(f),digits=digits),com=round(com,digits=digits),h2=round(h2,digits=digits))} else {
    results <- data.frame(round(unclass(f),digits=digits))}
+   
    results <- merge(results,contents,by="row.names",all.x=TRUE,sort=FALSE)
    rownames(results) <- results[,"Row.names"]
    results <- results[ord,-1]  #now put it back into the correct order
+   rownames(results) <- old.names
    if(!is.null(n)) {
      rn <-rownames(results)
      results <- cbind(results,rn)
@@ -369,6 +393,30 @@ if(is.null(names(fa.results)) )  {temp <- fa.results   #the matrix form
   "item.lookup" <- 
 function (f,m, dictionary,cut=.3, digits = 2) {
     f <- fa.sort(f)
+    none<- NULL   #A strange requirement of R 4.0
+     if(length(class(f)) > 1){ obnames <- cs(omega, fa, principal, iclust, none)
+     value <- inherits(f, obnames, which=TRUE)
+			   if (any(value > 1)) {value <- obnames[which(value >0)]} else {value <- "none"}} else {value <- "none"}
+  old.names <- NULL
+   switch(value,
+    
+    omega = {f <- f$schmid$sl
+            h2 <- NULL
+            old.names <- rownames(f)
+            rownames(f) <- sub("-","",old.names)},
+    fa    = {
+             h2 <- f$communality
+              com <- f$complexity
+              f <- f$loading        },
+    principal= {
+                h2 <- f$communality
+              com <- f$complexity
+              f <- f$loading},
+    iclust = {f <- f$loadings
+              h2 <- NULL},
+    none = {f <- f
+            h2 <- NULL})
+    
     if (!(is.matrix(f) || is.data.frame(f))) {
         h2 <- f$communality
         com <- f$complexity
@@ -398,14 +446,39 @@ function (f,m, dictionary,cut=.3, digits = 2) {
         sort = FALSE)
     rownames(results) <- results[, "Row.names"]
     results <- results[ord, -1]
-    res <- results[0,]  #make an empty data frame of the structure of results
-    for (i in 1:nfact) { temp <-results[abs(results[,i]) > cut,]
-     ord <- order(temp[,"means"])
-     res <- rbind(res,temp[ord,])
-     }
+    if(!is.null(old.names)) rownames(results) <- old.names
+    res <- results
+  #   res <- results[0,]  #make an empty data frame of the structure of results
+#     for (i in 1:nfact) { temp <- results[abs(results[,i]) > cut,]
+#      ord <- order(temp[,"means"])
+#      res <- rbind(res,temp[ord,])
+#      }
     return(res)
 }
 
+setCorLookup<- function(x,dictionary=NULL,cut=0,digits=2,p=.05) {
+coef <- x$coefficients
+probs <- x$Probability
+labels <- dictionary[rownames(coef),,drop=FALSE]
+
+coef[probs > p] <- NA
+result <- list()
+nvar <- NCOL(coef)
+for (i in 1:nvar) {
+ord <- order(abs(coef[,i]),decreasing=TRUE)
+temp <- cbind(coef=round(coef[ord,i],digits),labels[ord,])
+result[[i]] <- data.frame(temp[!is.na(temp[,1]),])
+}
+names(result) <- colnames(coef)
+result
+}
+
+
+"lookupItems" <- function(content=NULL,dictionary=NULL) {
+  location <-which(colnames(dictionary) %in% c("Item","Content","item"))
+  value <- grep(content,dictionary[,location])
+  dictionary[value,,drop=FALSE]
+}
 
 
 "falsePositive" <- function(sexy=.1,alpha=.05,power=.8) {
@@ -493,6 +566,8 @@ text(x,y-2,"Unreliable but Valid")
 # print(round(R,digits))
 # invisible(R)}
 
+#should we add in the abilty to not choke on character variables?
+
 "cor2" <- function(x,y=NULL,digits=2,use="pairwise",method="pearson") {
 multi <- FALSE
 if(is.list(x) && is.null(y)) {multi <- TRUE
@@ -524,23 +599,37 @@ pretty}
 
 
 #October 25, 2016
+#June 18 2020  added  as.factor to convert character strings that are not stored as factors
+#this has a downsize that it converts numbers stored as characters to factors (see nchar2numeric)
 "char2numeric" <- function(x) {
  nvar <- NCOL(x)
  for(i in 1:nvar) {   
         if(!is.numeric(x[[i]] ))  {
-                                  if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {  x[[i]] <- as.numeric(x[[i]]) 
+                                  if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {  x[[i]] <- as.numeric(as.factor(x[[i]])) 
+                          } else {x[[i]] <- NA} }
+              } 
+              invisible(x)} 
+#added June 25, 2020 to handle the case of numeric data stored as characters              
+"nchar2numeric" <- function(x) {
+ nvar <- NCOL(x)
+ for(i in 1:nvar) {   
+        if(!is.numeric(x[[i]] ))  { 
+                if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {
+                                  if(is.factor(unlist(x[[i]]))) {  x[[i]] <- as.numeric(as.factor(x[[i]])) } else {x[[i]] <- as.numeric(x[[i]])}
                           } else {x[[i]] <- NA} }
               } 
               invisible(x)} 
 
-#this just shows if it is a matrix
+
+#this just shows if it is a matrix is symmetric and has diagonals of 1
 "isCorrelation" <-  function(x) {value <- FALSE
   if(NROW(x) == NCOL(x)) {
   if( is.data.frame(x)) {if(isSymmetric(unname(as.matrix(x)))) { value <- TRUE}} else {if(isSymmetric(unname(x))) {value <- TRUE}}}
   value <- value && isTRUE(all.equal(prod(diag(as.matrix(x))),1) )
+  value <- value && isTRUE((min(x)>= -1) & (max(x) <= 1))
   return(value)}
   
-  #this just shows if it is a matrix
+  #this just shows if it is a symmetric matrix
 "isCovariance" <-  function(x) {value <- FALSE
   if(NROW(x) == NCOL(x)) {
   if( is.data.frame(x)) {if(isSymmetric(unname(as.matrix(x)))) { value <- TRUE}} else {if(isSymmetric(unname(x))) {value <- TRUE}}}
