@@ -6,9 +6,14 @@ if(!is.null(x$cfit.1)) {
  cat("\nVery Simple Structure of ", x$title,"\n") } else {cat("\nVery Simple Structure\n")} 
  cat("Call: ")
  print(x$call)
+ vss.max <- round(max(x$cfit.1) ,digits)
+ 
+ wh.max <- which.max(x$cfit.1)
+ if(wh.max !=first.max(x$vss.stats$cfit.1)) {cat("Although the VSS complexity 1 shows ",wh.max," factors, it is probably more reasonable to think about ",first.max(x$vss.stats$cfit.1), " factors\n")} else {
  cat("VSS complexity 1 achieves a maximimum of ")
- vss.max <- round(max(x$cfit.1) ,digits) 
- cat(vss.max," with " ,which.max(x$cfit.1), " factors\n") 
+ cat(vss.max," with " ,which.max(x$cfit.1), " factors\n")
+  }
+
  cat("VSS complexity 2 achieves a maximimum of ")
   vss.max <- round(max(x$cfit.2) ,digits) 
  cat(vss.max," with " ,which.max(x$cfit.2), " factors\n") 
@@ -35,7 +40,10 @@ if(!is.null(x$cfit.1)) {
  print(x$call)
  cat("VSS complexity 1 achieves a maximimum of ")
  vss.max <- round(max(x$vss.stats$cfit.1) ,digits) 
- cat(vss.max," with " ,which.max(x$vss.stats$cfit.1), " factors\n") 
+ wh.max <- which.max(x$vss.stats$cfit.1)
+ if(wh.max !=first.max(x$vss.stats$cfit.1)) {cat("Although the vss.max shows ",wh.max," factors, it is probably more reasonable to think about ",first.max(x$vss.stats$cfit.1), " factors\n")} else {
+ cat(vss.max," with " ,wh.max, " factors\n") }
+
  cat("VSS complexity 2 achieves a maximimum of ")
   vss.max <- round(max(x$vss.stats$cfit.2) ,digits) 
  cat(vss.max," with " ,which.max(x$vss.stats$cfit.2), " factors\n") 
@@ -58,3 +66,10 @@ if(!is.null(x$cfit.1)) {
  print(temp,digits=digits)}
    }  #end print.psych.vss
    
+"first.max" <- function(x) {
+ nx <- length(x) -1
+ first <- which.max(x)
+ for(i in 1:nx) {if(x[i] > x[i+1]){first <- i
+    break } } 
+    return(first)
+    }
\ No newline at end of file
